package cn.com.qmth.examcloud.service.core.api;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import io.swagger.annotations.ApiOperation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.service.core.entity.Permission;
import cn.com.qmth.examcloud.service.core.entity.RolePermissionRelation;
import cn.com.qmth.examcloud.service.core.service.RolePermissionRelationService;

@RestController
@RequestMapping("${app.api.root}/rolePermissionRelation")
public class RolePermissionRelationApi {
	
	@Autowired
	private RolePermissionRelationService rolePermissionRelationService;
	
	@ApiOperation(value="按系统和角色查询权限",notes="按系统和当前角色查询权限")
	@GetMapping("/permissions/{systemName}/{roleCodes}")
	public ResponseEntity findPermissionByRole(@PathVariable String systemName,@PathVariable String roleCodes){
		RolePermissionRelation rolePermissionRelation = new RolePermissionRelation();
		rolePermissionRelation.setSystemName(systemName);
		String[] roleCodeArray = roleCodes.split(",");
		List<String> roleCodeList = new ArrayList<String>();
		for(int i = 0;i<roleCodeArray.length;i++){
			roleCodeList.add(roleCodeArray[i]);
		}
		rolePermissionRelation.setRoleCodes(roleCodeList);
		Map<String,List<Permission>> permissionMap = rolePermissionRelationService.findPermissionByRole(rolePermissionRelation);
		return new ResponseEntity(permissionMap, HttpStatus.OK);
	}
	
	@ApiOperation(value="根据角色和系统查询数据",notes="根据角色和系统查询数据")
	@GetMapping("/{systemName}/{roleCode}/{permissionType}")
	public ResponseEntity findBySystemNameAndRoleName(@PathVariable String systemName,@PathVariable String roleCode,@PathVariable String permissionType){
		List<RolePermissionRelation> permissions = rolePermissionRelationService.findPermission(systemName,roleCode,permissionType);
		return new ResponseEntity(permissions, HttpStatus.OK);
	}
	
	@ApiOperation(value="保存角色权限设定",notes="保存角色权限设定")
    @PostMapping
	public ResponseEntity saveRolePermission(@RequestBody RolePermissionRelation rolePermissionRelation){
		rolePermissionRelationService.saveRolePermission(rolePermissionRelation);
		return new ResponseEntity(HttpStatus.OK);
	}
}
