package cn.com.qmth.examcloud.service.core.service;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import javax.transaction.Transactional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import cn.com.qmth.examcloud.common.dto.core.enums.CourseLevel;
import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.common.util.excel.ExcelReader;
import cn.com.qmth.examcloud.common.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.service.core.dto.CourseDto;
import cn.com.qmth.examcloud.service.core.entity.Course;
import cn.com.qmth.examcloud.service.core.entity.CourseSpeciatly;
import cn.com.qmth.examcloud.service.core.repo.CourseRepo;
import cn.com.qmth.examcloud.service.core.service.sync.DataSendService;

@Service
public class CourseService {

    @Autowired
    CourseRepo courseRepo;

	@Autowired
	DataSendService dataSendService;

    @Transactional
	public List<ExcelError> importCourse(Long orgId,InputStream inputStream) {
		List<Course> list = new ArrayList<Course>();
		ExcelReader excelReader = new ExcelReader(CourseDto.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
            @Override
            public ExcelError handle(Object obj) {
            	CourseDto dto = (CourseDto) obj;
            	Course course = new Course(dto.getName(),dto.getCode());
            	course.setOrgId(orgId);
            	course.setCreateTime(new Date());
				course.setEnable(true);
            	if("ZSB".equalsIgnoreCase(dto.getLevel())){
            		course.setLevel(CourseLevel.ZSB);
            	}
            	if("GQZ".equalsIgnoreCase(dto.getLevel())){
            		course.setLevel(CourseLevel.GQZ);
            	}
				if("GQB".equalsIgnoreCase(dto.getLevel())){
					course.setLevel(CourseLevel.GQB);
				}
            	if("ALL".equalsIgnoreCase(dto.getLevel())){
            		course.setLevel(CourseLevel.ALL);
            	}
                ExcelError error = importCheck(course);
                if (error == null) {
					addCourse(course);
                }
                return error;
            }
        });
		return excelErrors;
	}

	private void addCourse(Course course){
		Course old = courseRepo.findByOrgIdAndCode(course.getOrgId(),course.getCode());
		if(old == null){
			Course tempCourse = courseRepo.save(course);
			dataSendService.sendCourse(tempCourse);
		}else{
			old.setName(course.getName());
			old.setLevel(course.getLevel());
			old.setUpdateTime(new Date());
			Course tempCourse = courseRepo.save(old);
			dataSendService.sendCourse(tempCourse);
		}
	}

	private ExcelError importCheck(Course course) {
		if(StringUtils.isEmpty(course.getCode())){
			return new ExcelError("代码不能为空");
		}
		if(StringUtils.isEmpty(course.getName())){
			return new ExcelError("名称不能为空");
		}
		if(null == course.getLevel()){
			return new ExcelError("层次不能为空");
		}
		return null;
	}
	
	public Page<Course> findAll(Course courseCriteria,Long specialtyId, Pageable pageable) {
//		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
//				.withMatcher("name", startsWith())
//				.withMatcher("code", startsWith()).withIgnoreNullValues();
//		Example<Course> examExamStudentple = Example.of(orgCriteria,
//				exampleMatcher);
//		return courseRepo.findAll(examExamStudentple, pageable);
		Specification<Course> specification = getSpecification(courseCriteria,specialtyId);
		Page<Course> examStudents = courseRepo.findAll(specification,pageable);
		return examStudents;
	}

	private Specification<Course> getSpecification(Course courseCriteria,Long specialtyId) {
		Specification<Course> specification = (root, query, cb) -> {
		    List<Predicate> predicates = new ArrayList<>();
		    if(!StringUtils.isEmpty(courseCriteria.getOrgId())){
		    	predicates.add(cb.equal(root.get("orgId"),courseCriteria.getOrgId()));
		    }
		    if(!StringUtils.isEmpty(courseCriteria.getName())){
		    	predicates.add(cb.like(root.get("name"),"%"+courseCriteria.getName()+"%"));
		    }
		    if(!StringUtils.isEmpty(courseCriteria.getCode())){
		    	predicates.add(cb.like(root.get("code"),"%"+courseCriteria.getCode()+"%"));
		    }
		    if(!StringUtils.isEmpty(courseCriteria.getLevel())){
		    	predicates.add(cb.equal(root.get("level"),courseCriteria.getLevel()));
		    }
		    if(!StringUtils.isEmpty(courseCriteria.getEnable())){
		    	predicates.add(cb.equal(root.get("enable"),courseCriteria.getEnable()));
		    }
		    if(specialtyId!=null){
		    	Subquery<CourseSpeciatly> relationshipSubquery = query.subquery(CourseSpeciatly.class);
		    	Root<CourseSpeciatly> residencyRelationshipSubqueryRoot = relationshipSubquery.from(CourseSpeciatly.class);
		    	relationshipSubquery.select(residencyRelationshipSubqueryRoot);
		    	Predicate predicate = cb.equal(residencyRelationshipSubqueryRoot.get("specialtyId"), specialtyId);
		    	Predicate predicate2 = cb.equal(residencyRelationshipSubqueryRoot.get("courseId"), root.get("id"));
		    	relationshipSubquery.where(cb.and(predicate,predicate2));
		    	predicates.add(cb.exists(relationshipSubquery));
		    }
		    return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	public List<Course> findAll(Course courseCriteria) {
//		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
//				.withMatcher("name", startsWith())
//				.withMatcher("code", startsWith()).withIgnoreNullValues();
//        Example<Course> examExample = Example.of(orgCriteria,exampleMatcher);
//		List<Course> courseList = courseRepo.findAll(examExample);
//        return courseList;
		Specification<Course> specification = getSpecification(courseCriteria,null);
        return courseRepo.findAll(specification);
	}

	public Course save(Course course) throws Exception{
		checkCode(course.getOrgId(),course.getCode());
		course.setCreateTime(new Date());
		Course reCourse = courseRepo.saveAndFlush(course);
		dataSendService.sendCourse(reCourse);
		return reCourse;
	}

	private void checkCode(Long orgId,String code) {
		Course old = courseRepo.findByOrgIdAndCode(orgId,code);
		if(old!=null){
			throw new RuntimeException("代码已存在");
		}
	}

	@Transactional
	public void enableCourse(List<Long> courseIds, boolean enable) {
		for (Long courseId : courseIds) {
			Course course = courseRepo.findOne(courseId);
			course.setEnable(enable);
			course.setUpdateTime(new Date());
			Course reCourse = courseRepo.saveAndFlush(course);
			dataSendService.sendCourse(reCourse);
		}
	}
	
	@Transactional
	public void delete(String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s->Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        for(Long courseId:courseIds){
        	courseRepo.delete(courseId);
        }
	}

	public Course update(Long id, Course course) {
		Course old = courseRepo.findOne(id);
		if(!old.getCode().equals(course.getCode())){
			checkCode(course.getOrgId(),course.getCode());
		}
		course.setUpdateTime(new Date());
		Course reCourse = courseRepo.saveAndFlush(course);
		dataSendService.sendCourse(reCourse);
		return reCourse;
	}

}
