package cn.com.qmth.examcloud.service.core.service;

import cn.com.qmth.examcloud.service.core.entity.ExamSite;
import cn.com.qmth.examcloud.service.core.repo.ExamSiteRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import java.util.Date;
import java.util.List;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;
import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.exact;

@Service
public class ExamSiteService {

    @Autowired
	ExamSiteRepo examSiteRepo;


	public Page<ExamSite> findAll(ExamSite examSiteCriteria, Pageable pageable) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", contains())
				.withMatcher("code", contains())
                .withMatcher("orgId",exact());
		Example<ExamSite> examSiteExample = Example.of(examSiteCriteria,
				exampleMatcher);
		return examSiteRepo.findAll(examSiteExample, pageable);
	}

	public List<ExamSite> findAll(ExamSite examSiteCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", contains())
				.withMatcher("code", contains())
                .withMatcher("orgId",exact());
		Example<ExamSite> examSiteExample = Example.of(examSiteCriteria,
				exampleMatcher);
		return examSiteRepo.findAll(examSiteExample);
	}

	public ExamSite save(ExamSite examSite) throws Exception{
		checkCode(examSite.getOrgId(),examSite.getCode());
		examSite.setCreateTime(new Date());
		return examSiteRepo.save(examSite);
	}

	private void checkCode(Long orgId,String code) {
		ExamSite old = examSiteRepo.findFirstByOrgIdAndCode(orgId,code);
		if(old!=null){
			throw new RuntimeException("代码已存在");
		}
	}

	public ExamSite update(Long id, ExamSite examSite) {
		ExamSite old = examSiteRepo.findOne(id);
		if(!old.getCode().equals(examSite.getCode())){
			checkCode(examSite.getOrgId(),examSite.getCode());
		}
		examSite.setUpdateTime(new Date());
		return examSiteRepo.save(examSite);
	}

}
